@@ -114,13 +114,13 @@ inline qreal lookupOutlineIntensity(int intensity)
     case Breeze::InternalSettings::OutlineOff:
         return 0;
     case Breeze::InternalSettings::OutlineLow:
-        return 0.1;
+        return Breeze::Metrics::Bias_Default / 2;
     case Breeze::InternalSettings::OutlineMedium:
         return Breeze::Metrics::Bias_Default;
     case Breeze::InternalSettings::OutlineHigh:
-        return 0.4;
+        return Breeze::Metrics::Bias_Default * 2;
     case Breeze::InternalSettings::OutlineMaximum:
-        return 0.6;
+        return Breeze::Metrics::Bias_Default * 3;
     default:
         // Fallback to the Medium intensity.
         return Breeze::Metrics::Bias_Default;
@@ -149,7 +149,6 @@ Decoration::Decoration(QObject *parent, const QVariantList &args)
     , m_shadowAnimation(new QVariantAnimation(this))
 {
     g_sDecoCount++;
-    setDecorationName("breeze");
 }
 
 //________________________________________________________________
@@ -427,19 +426,16 @@ void Decoration::recalculateBorders()
     auto s = settings();
 
     // left, right and bottom borders
-    // HACK: for fractional scaling issues, this makes the borders slightly bigger
-    // on bottom and right side, which is then pushed under the window in kdecoration
-    const int fractionalScalingHack = outlinesEnabled() ? 1 : 0;
     const int left = isLeftEdge() ? 0 : borderSize();
-    const int right = isRightEdge() ? 0 : borderSize() + fractionalScalingHack;
-    const int bottom = (c->isShaded() || isBottomEdge()) ? 0 : borderSize(true) + fractionalScalingHack;
+    const int right = isRightEdge() ? 0 : borderSize();
+    const int bottom = (c->isShaded() || isBottomEdge()) ? 0 : borderSize(true);
 
     int top = 0;
     if (hideTitleBar()) {
         top = bottom;
     } else {
         QFontMetrics fm(s->font());
-        top += qMax(fm.height(), buttonHeight());
+        top += qMax(fm.height(), buttonSize());
 
         // padding below
         const int baseSize = s->smallSpacing();
@@ -490,14 +486,18 @@ void Decoration::updateButtonsGeometry()
     const auto s = settings();
 
     // adjust button position
-    const int bHeight = captionHeight() + (isTopEdge() ? s->smallSpacing() * Metrics::TitleBar_TopMargin : 0);
-    const int bWidth = buttonHeight();
-    const int verticalOffset = (isTopEdge() ? s->smallSpacing() * Metrics::TitleBar_TopMargin : 0) + (captionHeight() - buttonHeight()) / 2;
     const auto buttonList = m_leftButtons->buttons() + m_rightButtons->buttons();
     for (const QPointer<KDecoration2::DecorationButton> &button : buttonList) {
-        button.data()->setGeometry(QRectF(QPoint(0, 0), QSizeF(bWidth, bHeight)));
-        static_cast<Button *>(button.data())->setOffset(QPointF(0, verticalOffset));
-        static_cast<Button *>(button.data())->setIconSize(QSize(bWidth, bWidth));
+        auto btn = static_cast<Button *>(button.get());
+
+        const int verticalOffset = (isTopEdge() ? s->smallSpacing() * Metrics::TitleBar_TopMargin : 0);
+
+        const QSizeF preferredSize = btn->preferredSize();
+        const int bHeight = preferredSize.height() + verticalOffset;
+        const int bWidth = preferredSize.width();
+
+        btn->setGeometry(QRectF(QPoint(0, 0), QSizeF(bWidth, bHeight)));
+        btn->setPadding(QMargins(0, verticalOffset, 0, 0));
     }
 
     // left buttons
@@ -511,9 +511,11 @@ void Decoration::updateButtonsGeometry()
         if (isLeftEdge()) {
             // add offsets on the side buttons, to preserve padding, but satisfy Fitts law
             auto button = static_cast<Button *>(m_leftButtons->buttons().front());
-            button->setGeometry(QRectF(QPoint(0, 0), QSizeF(bWidth + hPadding, bHeight)));
-            button->setFlag(Button::FlagFirstInList);
-            button->setHorizontalOffset(hPadding);
+
+            QRectF geometry = button->geometry();
+            geometry.adjust(-hPadding, 0, 0, 0);
+            button->setGeometry(geometry);
+            button->setLeftPadding(hPadding);
 
             m_leftButtons->setPos(QPointF(0, vPadding));
 
@@ -532,11 +534,13 @@ void Decoration::updateButtonsGeometry()
         const int hPadding = s->smallSpacing() * Metrics::TitleBar_SideMargin;
         if (isRightEdge()) {
             auto button = static_cast<Button *>(m_rightButtons->buttons().back());
-            button->setGeometry(QRectF(QPoint(0, 0), QSizeF(bWidth + hPadding, bHeight)));
-            button->setFlag(Button::FlagLastInList);
-            // HACK: Add +1 to xpos because of the fractional scaling hack
-            // BUG: 481857
-            m_rightButtons->setPos(QPointF(size().width() - m_rightButtons->geometry().width() + 1, vPadding));
+
+            QRectF geometry = button->geometry();
+            geometry.adjust(0, 0, hPadding, 0);
+            button->setGeometry(geometry);
+            button->setRightPadding(hPadding);
+
+            m_rightButtons->setPos(QPointF(size().width() - m_rightButtons->geometry().width(), vPadding));
 
         } else {
             m_rightButtons->setPos(QPointF(size().width() - m_rightButtons->geometry().width() - hPadding - borderRight(), vPadding));
@@ -709,7 +713,7 @@ void Decoration::paintTitleBar(QPainter *painter, const QRect &repaintRegion)
 }
 
 //________________________________________________________________
-int Decoration::buttonHeight() const
+int Decoration::buttonSize() const
 {
     const int baseSize = m_tabletMode ? settings()->gridUnit() * 2 : settings()->gridUnit();
     switch (m_internalSettings->buttonSize()) {
@@ -730,6 +734,8 @@ int Decoration::buttonHeight() const
 void Decoration::onTabletModeChanged(bool mode)
 {
     m_tabletMode = mode;
+    Q_EMIT tabletModeChanged();
+
     recalculateBorders();
     updateButtonsGeometry();
 }
@@ -883,6 +889,9 @@ std::shared_ptr<KDecoration2::DecorationShadow> Decoration::createShadowObject(c
 
 void Decoration::setScaledCornerRadius()
 {
+    // On X11, the smallSpacing value is used for scaling.
+    // On Wayland, this value has constant factor of 2.
+    // Removing it will break radius scaling on X11.
     m_scaledCornerRadius = Metrics::Frame_FrameRadius * settings()->smallSpacing();
 }
 } // namespace
